import 'dart:collection';
import 'dart:convert';

import 'package:crypto/crypto.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/avd.dart';
// ignore: import_of_legacy_library_into_null_safe
import 'package:http/http.dart' as http;
import 'package:iotpaycreditcard/config/IOTPayConfig.dart';
import 'package:iotpaycreditcard/config/MyConfig.dart';
import 'package:iotpaycreditcard/generated/l10n.dart';

typedef IOTPayCallback = void Function(String result);

class DataProvider extends ChangeNotifier {
  http.Client _httpClient = http.Client();
  int _queryCnt = 0;
  int _maxQueryCnt = 30;
  bool _isBack = false;
  static double fontSize = 15.0;

  Queue _requestList = new Queue();
  bool _isTripleLine = true;
  bool get isBack => _isBack;
  bool get isTripleLine => _isTripleLine;
  set isTripleLine(value) {
    _isTripleLine = value;
    notifyListeners();
  }

  //define all the forms
  Map<String, Map<String, dynamic>> _formLists = {
    "credit": {
      "title": '',
      "items": [
        {
          "name": MyConfig.CARDNUMBER.name,
          "minLength": 8,
          "length": 20,
          "value": "",
          "defaultValue": "",
          "placeholder": S.current.CardNumber,
          "placeholdershort": S.current.CardNumberShort,
          "required": true,
          "inputType": TextInputType.phone,
          "inputWidth": 160.0,
          "inputLeft": 18.0,
          "inputRow": 0.0,
          "sufficIcon": IconButton(
              icon: AvdPicture.asset(
                "assets/images/card.xml",
                package: "iotpaycreditcard",
              ),
              onPressed: () {}),
          'txtEditingController': TextEditingController(),
          "fontSize": fontSize + 5.0,
          "fontWeight": FontWeight.bold,
        },
        {
          "name": MyConfig.HOLDER.name,
          "value": "",
          "defaultValue": "",
          "minLength": 1,
          "length": 30,
          "required": false,
          "placeholder": S.current.NameofHolder,
          "placeholdershort": S.current.NameofHolderShort,
          "inputType": TextInputType.text,
          "inputWidth": 50.0,
          "inputLeft": 185.0,
          "inputRow": 1.0,
          'txtEditingController': TextEditingController(),
          "fontSize": fontSize,
          //"fontStyle": FontStyle.normal,
        },
        {
          "name": MyConfig.EXPIRYDATE.name,
          "value": "",
          "defaultValue": "",
          "minLength": 4,
          "length": 4,
          "placeholder": S.current.MMYY,
          "placeholdershort": S.current.MMYY,
          "required": false,
          "inputType": TextInputType.phone,
          "inputWidth": 45.0,
          "inputLeft": 140.0,
          "inputRow": 2.0,
          'txtEditingController': TextEditingController(),
          "fontSize": fontSize - 2.0,
          //"fontStyle": FontStyle.normal,
        },
        {
          "name": MyConfig.CVV.name,
          "value": "",
          "defaultValue": "",
          "minLength": 3,
          "length": 4,
          "required": false,
          "placeholder": "CVV",
          "placeholdershort": "CVV",
          "inputType": TextInputType.number,
          "inputWidth": 30.0,
          "inputLeft": 220.0,
          "inputRow": 1.0,
          'txtEditingController': TextEditingController(),
          "fontSize": fontSize - 3.0,
          "fontStyle": FontStyle.italic,
        },
      ]
    },
    "credittype": {
      "title": '',
      "items": [
        {
          "code": MyConfig.AMERICAN_EXPRESS.name,
          "displayName": "American Express",
          "icon": IconButton(
              icon: AvdPicture.asset("assets/images/amex.xml",
                  package: "iotpaycreditcard"),
              onPressed: () {}),
          "cvcLength": 4,
          "cvcLabel": "CVC",
          "cardNumberLength": 15,
          "cardNumberPattern": RegExp(r"^(34|37)[0-9]*"),
          "partialPatterns": {2: RegExp(r"^(34|37)")}
        },
        {
          "code": MyConfig.DISCOVER.name,
          "displayName": "Discover",
          "icon": IconButton(
              icon: AvdPicture.asset("assets/images/discover.xml",
                  package: "iotpaycreditcard"),
              onPressed: () {}),
          "cardNumberPattern": RegExp(r"^(60|64|65)[0-9]*"),
          "partialPatterns": {2: RegExp(r"^(60|64|65)")}
        },
        {
          "code": MyConfig.JCB.name,
          "displayName": "JCB",
          "icon": IconButton(
              icon: AvdPicture.asset("assets/images/jcb.xml",
                  package: "iotpaycreditcard"),
              onPressed: () {}),
          "cardNumberPattern": RegExp(r"^(352[89]|35[3-8][0-9])[0-9]*"),
          "partialPatterns": {
            3: RegExp(r"^(352[89]|35[3-8][0-9])"),
            4: RegExp(r"^(352[89]|35[3-8][0-9])")
          }
        },
        {
          "code": MyConfig.DINER_ClUB.name,
          "displayName": "Diners Club",
          "icon": IconButton(
              icon: AvdPicture.asset("assets/images/diners.xml",
                  package: "iotpaycreditcard"),
              onPressed: () {}),
          "cardNumberPattern": RegExp(r"^(36|30|38|39)[0-9]*"),
          "partialPatterns": {2: RegExp(r"^(36|30|38|39)")}
        },
        {
          "code": MyConfig.VISA.name,
          "displayName": "Visa",
          "icon": IconButton(
              icon: AvdPicture.asset("assets/images/visa.xml",
                  package: "iotpaycreditcard"),
              onPressed: () {}),
          "cardNumberPattern": RegExp(r"^(4)[0-9]*"),
          "partialPatterns": {1: RegExp(r"^4")}
        },
        {
          "code": MyConfig.MASTERCARD.name,
          "displayName": "Mastercard",
          "icon": IconButton(
              icon: AvdPicture.asset("assets/images/mastercard.xml",
                  package: "iotpaycreditcard"),
              onPressed: () {}),
          "cardNumberPattern": RegExp(
              r"^(2221|2222|2223|2224|2225|2226|2227|2228|2229|222|223|224|225|226|227|228|229|23|24|25|26|270|271|2720|50|51|52|53|54|55|56|57|58|59|67)[0-9]*"),
          "partialPatterns": {
            2: RegExp(r"^(22|23|24|25|26|27|50|51|52|53|54|55|56|57|58|59|67)")
          }
        },
        {
          "code": MyConfig.UNIONPAY.name,
          "displayName": "UnionPay",
          "icon": IconButton(
              icon: AvdPicture.asset("assets/images/unionpay.xml",
                  package: "iotpaycreditcard"),
              onPressed: () {}),
          "cardNumberPattern": RegExp(r"^(62|81)[0-9]*"),
          "partialPatterns": {2: RegExp(r"^(62|81)")}
        },
      ]
    }
  };
  Map<String, Map<String, dynamic>> get formLists => _formLists;

  Future<dynamic> addCard(String secureId, String appKey) async {
    try {
      var url = MyConfig.URLROOT.name + MyConfig.URL_ADDCARD.name;
      var iOTPayCardInfo = getCardInfo(secureId, appKey);
      if (iOTPayCardInfo == null) {
        throw Exception("Please complete credit info");
      }

      return await httpResult(url, secureId, json.encode(iOTPayCardInfo))
          .catchError((e1) =>
              e1.toString() != "Exception: " + MyConfig.DUPENTRY.name
                  ? throw e1
                  : null);
    } catch (e) {
      throw e;
    }
  }

  getCardByCardNumber(String cardNumber) {
    for (int i = 4; i > 0; i--) {
      if (cardNumber.length < i) {
        continue;
      }
      var iotPayCreditCard = getCardByCardNumberSum(cardNumber, i);
      if (iotPayCreditCard != null) {
        return iotPayCreditCard;
      }
    }
    return {
      "code": MyConfig.UNKNOWN.name,
      "displayName": "Unknown",
      "icon": IconButton(
          icon: AvdPicture.asset(
            "assets/images/card.xml",
            package: "iotpaycreditcard",
          ),
          onPressed: () {})
    };
  }

  getCardByCardNumberSum(String cardNumber, int compareLength) {
    List<Map<String, Object>> listCards = _formLists['credittype']!['items'];
    for (int i = 0; i < listCards.length; i++) {
      Map<String, dynamic> ic = listCards[i];
      if (paternMatch(cardNumber, ic['cardNumberPattern'])) {
        return ic;
      }
    }
    var result;
    for (int i = 0; i < listCards.length; i++) {
      Map<String, dynamic> ic = listCards[i];
      String strShort = cardNumber.substring(0, compareLength);
      Map<int, RegExp> partialPatterns = ic["partialPatterns"];
      partialPatterns.forEach((key, value) {
        if (key == compareLength) {
          if (paternMatch(strShort, value)) {
            result = ic;
            //return ic;
          }
        }
      });
    }
    return result;
  }

  getCardCVV() {
    var item = _formLists["credit"]!["items"][3];
    if (!itemLengthValid(item)) {
      return null;
    }

    return item['value'];
  }

  getCardExpiryDate() {
    var item = _formLists["credit"]!["items"][2];
    if (!itemLengthValid(item)) {
      return null;
    }

    return item['value'];
  }

  getCardHolder() {
    var item = _formLists["credit"]!["items"][1];
    if (!itemLengthValid(item)) {
      return null;
    }

    return item['value'];
  }

  getCardNumber() {
    var item = _formLists["credit"]!["items"][0];
    if (!itemLengthValid(item)) {
      return null;
    }

    return item['value'];
  }

  getCardInfo(String secureId, String appKey) {
    var cardNo = getCardNumber();
    if (cardNo == null) {
      return null;
    }
    var holder = getCardHolder();
    if (holder == null) {
      return null;
    }
    var expiryDate = getCardExpiryDate();
    if (expiryDate == null) {
      return null;
    }
    var cvv = getCardCVV();
    if (cvv == null) {
      return null;
    }

    return {
      MyConfig.PARAMS_SECUREID.name: secureId,
      MyConfig.PARAMS_CARD_NUM.name: cardNo,
      MyConfig.PARAMS_HOLDER.name: holder,
      MyConfig.PARAMS_EXPIRYDATE.name: expiryDate,
      MyConfig.PARAMS_CVV.name: cvv,
      // ignore: unnecessary_null_comparison
      (appKey) != null ? MyConfig.SECRET.name : getSecrete(secureId, appKey): ''
    };
  }

  String hash(str) {
    var bytes1 = utf8.encode(str); // data being hashed
    var digest1 = sha256.convert(bytes1); // Hashing Process
    return digest1.toString();
  }

  getSecrete(secureId, appKey) {
    return hash(secureId + appKey);
  }

  String getFailedResult(dynamic e) {
    Map<String, String> result = {
      "retCode": MyConfig.FAIL.name,
      "retMsg": e.toString()
    };
    return json.encode(result);
  }

  double getWidth(item, contextWidth) {
    double width = item['inputWidth'].toDouble();
    if (isTripleLine) {
      if (item['name'] == MyConfig.CARDNUMBER.name) {
        width = contextWidth - 80.0;
      } else if (item['name'] == MyConfig.HOLDER.name) {
        width = 190.0;
      }
    }
    return width;
  }

  Future<dynamic> httpResult(var url, String secureId, String body) async {
    try {
      // ignore: unnecessary_null_comparison
      if (secureId == null || secureId.length < 1) {
        throw Exception("Please enter secureID");
      }
      _requestList.forEach((element) {
        if (element['url'] == url && element['secureId'] == secureId) {
          //print("====requestListExists true");
          throw Exception(MyConfig.DUPENTRY.name);
        }
      });
      //print("====requestListExists false");

      _requestList.add({"url": url, "secureId": secureId});

      var result = await _httpClient.post(url,
          body: body,
          headers: {'content-type': 'application/json'}).catchError((error) {
        throw error;
      });

      if (result.statusCode == 200) {
        //print("===result body:" + result.body);
        requestListRemove(url, secureId);
        return result.body.toString();
      } else {
        requestListRemove(url, secureId);
        throw Exception("Something went wrong. Please try again.");
      }
    } catch (e) {
      //print("==error: " + e.toString());
      throw e;
    }
  }

  bool itemLengthValid(item) {
    if (item['value'].length < item['minLength']) {
      return false;
    }
    if (item['value'].length > item['length']) {
      return false;
    }
    return true;
  }

  onChangeCardNumber(String value) {
    String cardNumber = retriveInt(value);
    String formatedCardNumber = "";
    int index = 0;
    for (int i = 0; i < cardNumber.length; i++) {
      var char = cardNumber[i];
      formatedCardNumber += char;
      index++;
      if (index == 4 && i < cardNumber.length - 1) {
        formatedCardNumber += ' ';
        index = 0;
      }
    }
    _formLists["credit"]!["items"][0]['txtEditingController']
      ..text = formatedCardNumber;
    _formLists["credit"]!["items"][0]['txtEditingController']
      ..selection = TextSelection.fromPosition(TextPosition(
          affinity: TextAffinity.downstream,
          offset: formatedCardNumber.length));
    //get card type
    var iotPayCreditCard = getCardByCardNumber(cardNumber);
    _formLists["credit"]!["items"][0]['sufficIcon'] = iotPayCreditCard['icon'];
    _formLists["credit"]!["items"][0]['value'] = cardNumber;

    notifyListeners();
  }

  onChangeCVV(String value) {
    String cvv = retriveInt(value);
    if (cvv.length > 4) {
      cvv = cvv.substring(0, 4);
    }
    _formLists["credit"]!["items"][3]['txtEditingController']..text = cvv;
    _formLists["credit"]!["items"][3]['txtEditingController']
      ..selection = TextSelection.fromPosition(
          TextPosition(affinity: TextAffinity.downstream, offset: cvv.length));
    //get card type
    _formLists["credit"]!["items"][3]['value'] = cvv;

    notifyListeners();
  }

  onChangeExpiryDate(String value) {
    String expiryDate = retriveInt(value);

    if (expiryDate.length > 0) {
      if (expiryDate[0] == '0') {
        if (expiryDate.length > 1 &&
            expiryDate[1] != '1' &&
            expiryDate[1] != '2' &&
            expiryDate[1] != '3' &&
            expiryDate[1] != '4' &&
            expiryDate[1] != '5' &&
            expiryDate[1] != '6' &&
            expiryDate[1] != '7' &&
            expiryDate[1] != '8' &&
            expiryDate[1] != '9') {
          expiryDate = expiryDate[0];
        }
      } else if (expiryDate[0] == '1') {
        if (expiryDate.length > 1 &&
            expiryDate[1] != '0' &&
            expiryDate[1] != '1' &&
            expiryDate[1] != '2') {
          expiryDate = expiryDate[0];
        }
      } else {
        expiryDate = "";
      }
    }
    if (expiryDate.length > 4) {
      expiryDate = expiryDate.substring(0, 4);
    }
    String formatedExpiryDate = "";
    for (int i = 0; i < expiryDate.length; i++) {
      var char = expiryDate[i];
      formatedExpiryDate += char;
      if (i == 1 && i < expiryDate.length - 1) {
        //check if mm is correct

        formatedExpiryDate += '/';
      }
    }

    _formLists["credit"]!["items"][2]['txtEditingController']
      ..text = formatedExpiryDate;
    _formLists["credit"]!["items"][2]['txtEditingController']
      ..selection = TextSelection.fromPosition(TextPosition(
          affinity: TextAffinity.downstream,
          offset: formatedExpiryDate.length));
    if (formatedExpiryDate.length == 5) {
      _formLists["credit"]!["items"][2]['value'] =
          formatedExpiryDate.replaceAll(new RegExp(r'[^0-9]'), '');
    } else {
      _formLists["credit"]!["items"][2]['value'] = '';
    }
  }

  onChangeHolder(String value) {
    _formLists["credit"]!["items"][1]['value'] = value;
  }

  void onChange(String eventObj, String value) {
    //print("changed for " + eventObj + ", value is " + value);
    if (eventObj == MyConfig.CARDNUMBER.name) {
      onChangeCardNumber(value);
    } else if (eventObj == MyConfig.EXPIRYDATE.name) {
      onChangeExpiryDate(value);
    } else if (eventObj == MyConfig.CVV.name) {
      onChangeCVV(value);
    } else if (eventObj == MyConfig.HOLDER.name) {
      onChangeHolder(value);
    }
  }

  void onTab(String eventObj, double screenWidth) {
    //print("tab for " + eventObj);
    if (eventObj == MyConfig.HOLDER.name) {
      _formLists["credit"]!["items"][0]['inputWidth'] = 115.0;
      _formLists["credit"]!["items"][1]['inputWidth'] = 100.0;
      _formLists["credit"]!["items"][1]['inputLeft'] = 135.0;
    } else if (eventObj == MyConfig.CARDNUMBER.name) {
      _formLists["credit"]!["items"][0]['inputWidth'] = 160.0;
      _formLists["credit"]!["items"][1]['inputWidth'] = 50.0;
      _formLists["credit"]!["items"][1]['inputLeft'] = 185.0;
    }

    if (eventObj == MyConfig.CVV.name) {
      _isBack = true;
    } else {
      _isBack = false;
    }
    notifyListeners();
  }

  bool paternMatch(String value, RegExp pattern) {
    return pattern.hasMatch(value);
  }

  Future<dynamic> pay(String secureId, String appKey) async {
    try {
      String url = MyConfig.URLROOT.name + MyConfig.URL_PURCHASE.name;
      var iOTPayCardInfo = getCardInfo(secureId, appKey);
      if (iOTPayCardInfo == null) {
        throw Exception("Please complete credit info");
      }

      return await httpResult(url, secureId, json.encode(iOTPayCardInfo))
          .catchError((e1) =>
              e1.toString() != "Exception: " + MyConfig.DUPENTRY.name
                  ? throw e1
                  : null);
    } catch (e) {
      throw e;
    }
  }

  queryOrder(dynamic value, iotPayCallback, String secureId) async {
    //value to json
    //print("===begin to query Order at " + _queryCnt.toString());
    Map<String, dynamic> iotPayRes = json.decode(value);
    // ignore: unnecessary_null_comparison
    if (iotPayRes != null //if order status is unknown, do order query.
        &&
        iotPayRes["retCode"] != null &&
        iotPayRes["retCode"] == MyConfig.SUCCESS.name &&
        iotPayRes["retData"] != null &&
        iotPayRes["retData"]["status"] != null &&
        iotPayRes["retData"]["status"] != 2 &&
        iotPayRes["retData"]["status"] != 3 &&
        _queryCnt > 0) {
      _queryCnt--;
      //print("=====status is " + iotPayRes["retData"]["status"]);
      //call order query afer one second
      await Future.delayed(Duration(seconds: 1)); //等1秒;
      Map<String, String> params = {MyConfig.PARAMS_SECUREID.name: secureId};
      httpResult(MyConfig.URLROOT.name + MyConfig.URL_QUERYORDER.name, secureId,
              json.encode(params))
          .then((valueNew) => valueNew != null
              ? queryOrder(valueNew, iotPayCallback, secureId)
              : null)
          .catchError((e1) =>
              e1.toString() != "Exception: " + MyConfig.DUPENTRY.name
                  ? throw e1
                  : null);
    } else {
      _queryCnt = 0;
      iotPayCallback(value.toString());
    }
  }

  requestListRemove(var url, String secureId) {
    _requestList.removeWhere(
        (element) => element['url'] == url && element['secureId'] == secureId);
    print("====requestList removed");
  }

  retriveInt(String value) {
    return value.replaceAll(new RegExp(r'[^0-9]'), '');
  }

  sendRequest(String secureId, dynamic layoutType, String appKey,
      IOTPayCallback iotPayCallback) {
    try {
      // ignore: unnecessary_null_comparison
      if (secureId == null || secureId.length < 1) {
        throw Exception("Please enter secureID");
      }
      Future<dynamic> result;
      if (layoutType == IOTPayConfig.AddCard) {
        result = addCard(secureId, appKey);
        result
            .then((value) =>
                value != null ? iotPayCallback(value.toString()) : null)
            .catchError((e) {
          iotPayCallback(getFailedResult(e));
        });
      } else if (layoutType == IOTPayConfig.SimplePurchase) {
        result = pay(secureId, appKey);
        _queryCnt = _maxQueryCnt;
        result
            .then((value) => value != null
                ? queryOrder(value, iotPayCallback, secureId)
                : null)
            .catchError((e) => iotPayCallback(getFailedResult(e)));
      }
    } catch (e) {
      iotPayCallback(getFailedResult(e));
    }
  }
}
